//
// FileEntry.cs
//
// Author:
//   Lluis Sanchez Gual
//
// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//


using System;
using System.Collections;
using MonoDevelop.Projects;
using MonoDevelop.Projects.Parser;

namespace MonoDevelop.Projects.Parser
{	
	[Serializable]
	class FileEntry
	{
		string filePath;
		DateTime parseTime;
		ClassEntry firstClass;
		int parseErrorRetries;
		
		[NonSerialized]
		bool disableParse;
		
		[NonSerialized]
		bool inParseQueue;
		
		public FileEntry (string path)
		{
			filePath = path;
			parseTime = DateTime.MinValue;
		}
		
		public string FileName
		{
			get { return filePath; }
		}
		
		public bool DisableParse {
			get { return disableParse; }
			set { disableParse = value; }
		}
		
		public bool InParseQueue {
			get { return inParseQueue; }
			set { inParseQueue = value; }
		}
		
		public DateTime LastParseTime
		{
			get { return parseTime; }
			set { parseTime = value; }
		}
		
		public ClassEntry FirstClass
		{
			get { return firstClass; }
		}
		
		public int ParseErrorRetries
		{
			get { return parseErrorRetries; }
			set { parseErrorRetries = value; }
		}
		
		public void SetClasses (ArrayList list)
		{
			firstClass = null;
			foreach (ClassEntry ce in list)
				AddClass (ce);
		}
		
		public void AddClass (ClassEntry ce)
		{
			if (ce.FileEntry != null)
				ce.FileEntry.RemoveClass (ce);
				
			ce.NextInFile = firstClass;
			firstClass = ce;
		}
		
		public void RemoveClass (ClassEntry ce)
		{
			ClassEntry oldent = null;
			ClassEntry curent = firstClass;
			
			while (curent != null && curent != ce) {
				oldent = curent;
				curent = curent.NextInFile;
			}
			
			if (curent == null) 
				return;
			else if (oldent == null)
				firstClass = curent.NextInFile;
			else
				oldent.NextInFile = curent.NextInFile;
				
			ce.FileEntry = null;
		}
		
		public bool IsAssembly
		{
			get { return filePath.ToLower().EndsWith (".dll"); }
		}
	}
}
